# SLICE CLASSES FOR MOISTDIFF
#
# CLASS WEATHER
#
class Weather:
  ''' Sets a relative humidity depending on the elapsed time'''

  def __init__(self,sIndex,wType,cycleTime,offset,averageRH,swingRH,tture,windSpeed):
    self.sliceType = 0 # weather
    self.sIndex = sIndex # index in slice list or array
    self.wType = int(wType) #squarewave=0 sinewave=1 readfromfile=2
    self.cycleTime = cycleTime #period in hours
    self.offset = offset #offset into period in hours
    self.averageRH = averageRH #RH as fraction or average flux in kg
    self.swingRH = swingRH #RH fractional excursion of flux excursion in kg
    self.tture = tture # temperature
    self.windSpeed = windSpeed # m/s
    self.rh = 0.5 #dummy value
    self.vp = 1000 #dummy
    self.waterPerKg = 10 #dummy
    self.dummy = 1 #dummy used to complete arg list

  def diffuse(self, elapsedTime,repRate,prevSlice):
    pass #empty function

  def update(self,backwash,elapsedTime): #backwash not used
    import math, Cmath
    elapsedCycles = (elapsedTime + self.offset)/self.cycleTime
    fractionIntoCycle = elapsedCycles - int(elapsedCycles)
    if (self.wType == 0): #square wave
      if (fractionIntoCycle < 0.5):
        self.rh = self.averageRH + self.swingRH
      else:
        self.rh = self.averageRH - self.swingRH
    if (self.wType == 1): # sine wave
      self.rh = self.averageRH + self.swingRH * math.sin(fractionIntoCycle * 2 * math.pi)
    self.vp = Cmath.RHandTtoVP(self.rh, self.tture)
    self.waterPerKg = Cmath.VPtoKGKG(self.vp)
    

  def showStatus(self):
    print 'WEATHER: ',
    #print 'slice type=', self.sliceType,
    print 'slice index=', self.sIndex,
    print 'weather type=', self.wType,
    print 'cycle time=', self.cycleTime,
    print 'offset=', self.offset,
    print 'average RH=', self.averageRH,
    print 'swing RH=', self.swingRH,
    print 't=', self.tture,
    print 'wind speed=', self.windSpeed,
    print 'RH=', self.rh,
    print 'vp=', self.vp
    print ' '

  def showChanges(self):
    print 'WEATHER ',
    print 'RH:',
    print ' %3.1f'% (self.rh*100),
    print 'VP:',
    print ' %3.1f'% self.vp
   
# end of weather class definition
# ---------------------------------------------------------

# CLASS ROOM
#  
class Room:
  ''' leaks in air from weather and adds some moisture itself,
      as defined by iWaterFlux, iWaterSwing, cTime (cycle time is independent of weather)
      offset and cType, triangular or sine. No square wave is programmed
  '''
  def __init__(self,sIndex,leakrate,rh,tture,thick,windspeed,iWaterFlux,iWaterSwing,cTime,offset,cType): 
    import Cmath   
    self.sliceType = 1 # room
    self.sIndex = sIndex
    self.leakrate = leakrate
    self.rh = rh
    self.tture = tture
    self.thick = thick
    #self.windspeed = float(windspeed) * 1E-12
    self.fluxIn = 0 #water from weather leaking in
    self.iWaterFlux = iWaterFlux
    self.iWaterSwing = iWaterSwing
    self.cTime = cTime
    self.offset = offset
    self.cType = int(cType) #triangular=0 or sine=1 
    self.vp = Cmath.RHandTtoVP(self.rh, self.tture)
    self.waterPerKg = Cmath.VPtoKGKG(self.vp)
    self.water = thick * Cmath.VPtoKgM3( self.vp, self.tture)
    self.wDiffCo = windspeed * 1E-12 #refine later to make a variable boundary layer diffco.
    self.targetWeight = 0 #weight of reservoir
    self.prevWt = 0 #previous reservoir weight
    self.dummy = 1 #dummy used to complete arg list

  def diffuse(self, elapsedTime,repRate,prevSlice):
    #need to add code here if 'room' is space within wall
    #calc. water coming from weather
    #volume of outside air injected is timeinterval in hours x 
    #leakrate x roomvolume(= depth). The same amount of air leaks out
    import Cmath, math
    airX = repRate/3600 * self.leakrate * self.thick #amount of air leaking in
    waterIn = airX * Cmath.VPtoKgM3(prevSlice.vp, self.tture)
    waterOut = airX * Cmath.VPtoKgM3(self.vp,self.tture)
    self.fluxIn = waterIn - waterOut
    ##Internally generated water
    #elapsedCycles =(elapsedTime + self.offset)/self.cTime
    #fractionIntoCycle = elapsedCycles - int(elapsedCycles)
    #self.prevWt = self.targetWeight #store previous amount of water in reservoir
    #if(self.cType == 0): #triangular wave
    #  if(fractionIntoCycle < 0.25):
    #      self.targetWeight = self.iWaterSwing * fractionIntoCycle * 4
    #  if((fractionIntoCycle > 0.25) & (fractionIntoCycle < 0.75)):
    #      self.targetWeight = self.iWaterSwing * (0.5 - fractionIntoCycle)* 4
    #  if(fractionIntoCycle > 0.75):
    #  	  self.targetWeight = self.iWaterSwing * (fractionIntoCycle - 1)* 4
    #if(self.cType == 1): #sine wave
	#  #matches climate chamber operation with flux swing over 1/4 cycle:
    #  #NOTE: only works with no constant flux!
    #    self.targetWeight = self.iWaterSwing * math.sin(fractionIntoCycle * 2 *math.pi)
    #self.iWaterFlux = self.targetWeight - self.prevWt
    prevSlice.update(self.fluxIn,elapsedTime) # first param ignored by weather slice

  def update(self, backwash,elapsedTime):
    import Cmath
    self.water += self.fluxIn #from weather
    #self.water += self.iWaterFlux #from within room
    self.water -= backwash #to wall
    kgPerCuM = self.water/self.thick;
    self.vp = Cmath.KgM3toVP(kgPerCuM, self.tture)
    self.waterPerKg = Cmath.VPtoKGKG(self.vp)      
    self.rh = Cmath.VPtoRH(self.vp,self.tture)
    #self.showStatus()

  def showStatus(self):
    print 'ROOM',
    #print 'slice type=', self.sliceType,
    print 'slice index=', self.sIndex,
    print 'leak rate=', self.leakrate,
    print 't=', self.tture,
    print 'thickness=', self.thick,
    print '"diff. coeff."=', self.wDiffCo,
    print 'water flux in=', self.iWaterFlux,
    print 'water swing in=', self.iWaterSwing,
    #print 'cycle time=', self.cTime,
    print 'offset=', self.offset,
    #print 'flux cycle type=', self.cType,
    print 'RH=', self.rh,
    print 'vp=', self.vp

  def showChanges(self):
    print 'ROOM',
    print 'RH:',
    print '%3.1f'% (self.rh*100),
    print 'flux in:',
    print ' %3.4f'% (self.fluxIn*100000)
      
# end of room class definition
#-------------------------------------------

#CLASS WALL
#
class Wall:

 def __init__(self,sIndex,thick,density,waterCap,waterDiffCo,waterPerKg,tture):
   import Cmath   
   self.sliceType = 3 # wall
   self.sIndex = sIndex # index in slice list or array
   self.thick = thick #m
   self.density = density #kg/m3
   self.waterCap = waterCap #kg/kg per 100%RH change
   self.wDiffCo = waterDiffCo * 1E-12
   self.waterPerKg = waterPerKg
   self.tture = tture
   self.water = self.waterPerKg * self.density * self.thick
   self.rh = self.waterPerKg / self.waterCap
   self.vp = Cmath.RHandTtoVP(self.rh, self.tture)
   self.fluxIn = 0 #flux flowing in from previous slice
   self.dummy = 1 #dummy used to complete arg list

 def diffuse(self, elapsedTime, repRate, prevSlice):
     if (prevSlice.sliceType == 1): # previous slice is a room type
       #use its resistance, cast into a diff. coeff.
       #to calculate flux into this slice
       self.fluxIn = repRate * prevSlice.wDiffCo * (prevSlice.vp - self.vp)
     else:
       #calc flux into slice on the basis of this slice's diffusion coeff.
       self.fluxIn = repRate * self.wDiffCo * (prevSlice.vp - self.vp)/self.thick
     prevSlice.update(self.fluxIn,self.dummy);  

 def update(self, backwash,elapsedTime):
      import Cmath
      #update water content
      self.water += self.fluxIn #add flux flowing in from upstream
      self.water -= backwash #subtract flux flowing out downstream
      #now recalc all variables
      self.waterPerKg = self.water/(self.thick * self.density)
      self.rh = self.waterPerKg / self.waterCap
      self.vp = Cmath.RHandTtoVP(self.rh, self.tture)
      #self.showStatus() #testing

 def showStatus(self):
    print 'WALL: ',
    print 'slice type=', self.sliceType,
    print 'slice index=',    self.sIndex,
    print 'thickness=',    self.thick,
    print 'density=',    self.density
    print ' water capacity=',    self.waterCap,
    print 'water diff coeff=',    self.wDiffCo,
    print 'water per kg=',    self.waterPerKg,
    print 'temperature=',    self.tture,
    print 'RH=',self.rh,
    print 'vp=',self.vp
    print ' '

 def showChanges(self):
    print 'WALL slice ',
    print '%d' % self.sIndex,
    print 'RH:',
    print '%3.1f'% (self.rh*100),
    print 'flux in:',
    print '%3.4f'% (self.fluxIn*100000)
 
# end of class wall definition
#-----------------------------------------------
